#editor {
    width: 100%;
    height: 300px;
}

/* CSS By Pygents */
.hll{background-color:#ffc}.c{color:#408080;font-style:italic}.err{border:1px solid red}.k{color:green;font-weight:700}.o{color:#666}.ch,.cm{color:#408080;font-style:italic}.cp{color:#bc7a00}.c1,.cpf,.cs{color:#408080;font-style:italic}.gd{color:#a00000}.ge{font-style:italic}.gr{color:red}.gh{color:navy;font-weight:700}.gi{color:#00a000}.go{color:#888}.gp{color:navy}.gp,.gs{font-weight:700}.gu{color:purple;font-weight:700}.gt{color:#04d}.kp{color:green}.kc,.kd,.kn,.kr{color:green;font-weight:700}.kt{color:#b00040}.m{color:#666}.s{color:#ba2121}.na{color:#7d9029}.nb{color:green}.nc{color:#00f;font-weight:700}.no{color:#800}.nd{color:#a2f}.ni{color:#999}.ne,.ni{font-weight:700}.ne{color:#d2413a}.nf{color:#00f}.nl{color:#a0a000}.nn{color:#00f}.nn,.nt{font-weight:700}.nt{color:green}.nv{color:#19177c}.ow{color:#a2f;font-weight:700}.w{color:#bbb}.mb,.mf,.mh,.mi,.mo{color:#666}.dl,.sa,.sb,.sc,.sd{color:#ba2121}.sd{font-style:italic}.s2{color:#ba2121}.se{color:#b62;font-weight:700}.sh{color:#ba2121}.si{color:#b68;font-weight:700}.sx{color:green}.sr{color:#b68}.s1{color:#ba2121}.ss{color:#19177c}.bp{color:green}.fm{color:#00f}.vc,.vg,.vi,.vm{color:#19177c}.il{color:#666}